//=============================================================================
// Massive lightning gun.
// http://mrevil.pwp.blueyonder.co.uk/unreal/
//=============================================================================
class SentinelWeapon_ElectronLance extends SentinelWeapon;

/** FireInfo for primary arcs fired directly forward. */
var() SentinelWeaponComponent_FireInfo PrimaryArcFireInfo;
/** Number of main "projectiles" to spawn for each shot. */
var() int NumPrimaryArcs;

/** FireInfo for secondary arcs fired in all direction. */
var() SentinelWeaponComponent_FireInfo SecondaryArcFireInfo;
/** Number of additional arcs to spawn. */
var() int NumSecondaryArcs;

var() array<SentinelWeaponComponent_MuzzleFlash> SparkGapFlashes;
/** Names of sockets located at spark gaps. */
var() array<name> SparkGapSocketNames;

/** An arc trace that misses a pawn by this much or less will be drawn to the pawn. */
var() float CosAttractionAngle;

simulated function InitMuzzleFlash()
{
	local int i;

	super.InitMuzzleFlash();

	if(Cannon != none)
	{
		for(i = 0; i < SparkGapFlashes.length; i++)
		{
			SparkGapFlashes[i].Initialize(Cannon.WeaponComponent, SparkGapSocketNames[i]);
		}
	}
}

simulated function FlashMuzzleFlash()
{
	local int i;

	super.FlashMuzzleFlash();

	for(i = 0; i < SparkGapFlashes.length; i++)
	{
		SparkGapFlashes[i].Flash();
	}
}

function bool FireAt(Vector Start, Rotator BarrelDir, Vector End)
{
	FireInfo = SecondaryArcFireInfo;
	SpawnArcs(Start, End, NumSecondaryArcs);

	FireInfo = PrimaryArcFireInfo;
	SpawnArcs(Start, End, NumPrimaryArcs);

	super.FireAt(Start, BarrelDir, End);

	return true;
}

function SpawnArcs(Vector Start, Vector End, int NumArcs)
{
	local Vector TraceStart, TraceEnd;
	local Actor HitActor;
	local Pawn HitPawn;
	local Vector HitLocation, HitNormal;
	local SentinelProjectile_ElectronLance Proj;
	local int i;

	TraceStart = Start;

	for(i = 0; i < NumArcs; i++)
	{
		TraceEnd = Normal(End - Start) + (Normal(VRand()) * FireInfo.Spread);
		TraceEnd = Normal(TraceEnd);
		TraceEnd *= FireInfo.MaxRange;
		TraceEnd += TraceStart;

		HitActor = Cannon.Trace(HitLocation, HitNormal, TraceEnd, TraceStart, true, vect(0.0, 0.0, 0.0),, TRACEFLAG_Bullet);

		if(HitActor == none)
		{
			HitLocation	= TraceEnd;
			HitNormal = Normal(TraceStart - TraceEnd);
		}
		else
		{
			HitPawn = Pawn(HitActor);
		}

		//If the trace didn't hit a pawn, maybe there is one close by to arc to instead.
		if(HitPawn == none)
		{
			HitPawn = ClosestPawnToLineAngular(TraceStart, Normal(TraceEnd - TraceStart), FMin(VSize(TraceEnd - TraceStart) + 100.0, FireInfo.MaxRange), CosAttractionAngle);

			if(HitPawn != none)
			{
				HitLocation = HitPawn.Location;
				HitNormal = Normal(TraceStart - HitPawn.Location);
				HitActor = HitPawn;
			}
		}

		Proj = SentinelProjectile_ElectronLance(Cannon.Spawn(FireInfo.ProjectileClass, Cannon,, HitLocation, Rotator(HitNormal)));

		if(Proj != none)
		{
			Proj.Lance = self;
			Proj.Cannon = Cannon;
			Proj.Range = FireInfo.MaxRange;

			if(HitActor != none)
			{
				Proj.SetBase(HitActor);
			}
		}
	}
}

/**
 * Finds the pawn the smallest angle from the given line, within the given limit.
 *
 * @param	LineStart		start of line
 * @param	LineDir			direction of line
 * @param	MaxDistance		pawns further than this distance from LineStart will be ignored
 * @param	MaxCosAngle		pawns more than arccos this angle from the line will be ignored
 */
final function Pawn ClosestPawnToLineAngular(Vector LineStart, Vector LineDir, float MaxDistance, float MaxCosAngle)
{
	local Pawn P, ClosestPawn;
	local Vector PawnDir;
	local float CosAngle, ClosestCosAngle;

	ClosestCosAngle = MaxCosAngle;

	foreach WorldInfo.AllPawns(class'Pawn', P, LineStart, MaxDistance)
	{
		if(P.IsValidTargetFor(Cannon.InstigatorController))
		{
			PawnDir = Normal(P.Location - LineStart);
			CosAngle = LineDir Dot PawnDir;

			if(CosAngle > ClosestCosAngle)
			{
				if(FastTrace(P.Location, LineStart,, true))
				{
					ClosestPawn = P;
					ClosestCosAngle = CosAngle;
				}
			}
		}
	}

	return ClosestPawn;
}

defaultproperties
{
	FriendlyName="Electron Lance"
	Description="Technical details unavailable."

	NumPrimaryArcs=4
	NumSecondaryArcs=10
	CosAttractionAngle=0.9

	WeaponMesh=SkeletalMesh'SentinelGear_Resources.Meshes.Weapons.SK_ElectronLance'
	PhysicsAsset=PhysicsAsset'SentinelGear_Resources.Physics.SK_ElectronLance_Physics'
	WeaponMaterial=MaterialInstance'SentinelGear_Resources.Materials.Weapons.MI_ElectronLance'
	WeaponSpawnMaterial=MaterialInstance'SentinelGear_Resources.Materials.Weapons.MI_ElectronLance_Spawn'
	WeaponDeadMaterial=MaterialInstance'SentinelGear_Resources.Materials.Weapons.MI_ElectronLance_Dead'
	WeaponScale=2.5

	ActivateSound=SoundCue'SentinelGear_Resources.Sounds.Weapons.SniperRiflePickupCue'

	ExtraStrength=0.9
	AmmoCost=280
	CompatibleSentinels.Remove(class'Sentinel_Ceiling')

	Begin Object Class=SentinelWeaponComponent_FireInfo Name=FireInfo0
		FireInterval=7.5
		FireOffset=(X=60.0,Y=0.0,Z=0.0)
		ProjectileClass=class'SentinelProjectile_ElectronLance'
		MaxRange=7000.0
		Spread=0.1
		FireSound=SoundCue'SentinelGear_Resources.Sounds.Weapons.ElectronLance_FireCue'
		RotationStopTime=2.0
	End Object
	FireInfo=FireInfo0
	PrimaryArcFireInfo=FireInfo0

	Begin Object Class=SentinelWeaponComponent_FireInfo Name=FireInfo1
		FireInterval=7.5
		FireOffset=(X=60.0,Y=0.0,Z=0.0)
		ProjectileClass=class'SentinelProjectile_ElectronLance'
		MaxRange=4000.0
		Spread=1.0
		FireSound=SoundCue'SentinelGear_Resources.Sounds.Weapons.ElectronLance_FireCue'
		RotationStopTime=2.0
	End Object
	SecondaryArcFireInfo=FireInfo1

	Begin Object Class=SentinelWeaponComponent_MuzzleFlashLight Name=MuzzleFlashLightComponent0
		LightOffset=(X=80.0,Y=0.0,Z=0.0)
		TimeShift=((StartTime=0.0,Radius=500,Brightness=5,LightColor=(R=255,G=100,B=255,A=255)),(StartTime=1.0,Radius=300,Brightness=5,LightColor=(R=100,G=200,B=255,A=255)),(StartTime=2.5,Radius=100,Brightness=0,LightColor=(R=0,G=128,B=255,A=255)))
	End Object
	MuzzleFlashLight=MuzzleFlashLightComponent0

	Begin Object Class=SentinelWeaponComponent_MuzzleFlash Name=MuzzleFlash0
		Template=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_ElectronLance_SparkGap'
		MuzzleFlashDuration=2.0
	End Object
	SparkGapFlashes[0]=MuzzleFlash0
	SparkGapSocketNames[0]=SparkGap0

	Begin Object Class=SentinelWeaponComponent_MuzzleFlash Name=MuzzleFlash1
		Template=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_ElectronLance_SparkGap'
		MuzzleFlashDuration=2.0
	End Object
	SparkGapFlashes[1]=MuzzleFlash1
	SparkGapSocketNames[1]=SparkGap1

	Begin Object Class=SentinelWeaponComponent_MuzzleFlash Name=MuzzleFlash2
		Template=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_ElectronLance_SparkGap'
		MuzzleFlashDuration=2.0
	End Object
	SparkGapFlashes[2]=MuzzleFlash2
	SparkGapSocketNames[2]=SparkGap2

	Begin Object Class=SentinelWeaponComponent_MuzzleFlash Name=MuzzleFlash3
		Template=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_ElectronLance_SparkGap'
		MuzzleFlashDuration=2.0
	End Object
	SparkGapFlashes[3]=MuzzleFlash3
	SparkGapSocketNames[3]=SparkGap3

	Begin Object Class=SentinelWeaponComponent_MuzzleFlash Name=MuzzleFlash4
		Template=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_ElectronLance_SparkGap'
		MuzzleFlashDuration=2.0
	End Object
	SparkGapFlashes[4]=MuzzleFlash4
	SparkGapSocketNames[4]=SparkGap4

	Begin Object Class=SentinelWeaponComponent_MuzzleFlash Name=MuzzleFlash5
		Template=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_ElectronLance_SparkGap'
		MuzzleFlashDuration=2.0
	End Object
	SparkGapFlashes[5]=MuzzleFlash5
	SparkGapSocketNames[5]=SparkGap5
}
