//=============================================================================
// Gibs, in an instant.
// http://mrevil.pwp.blueyonder.co.uk/unreal/
//=============================================================================
class SentinelWeapon_MicroShock extends SentinelWeapon;

var() SentinelWeaponComponent_HitEffects HitEffects;

/** Beam will be steered this proportion away from the actual barrel direction towards perfect aim i.e. 1.0 => never misses, 0.0 => misses quite often. */
var() float Precision;

/** Effect to use for beam. */
var() ParticleSystem BeamTemplate;
var Emitter BeamEmitter;
/** How long to wait before moving emitter to its end. */
var() float BeamMoveDelay;
var Vector BeamEndLocation;
/** Light to spawn at end of beam. */
var class<UTExplosionLight> ImpactLightClass;

function bool FireAt(Vector Start, Rotator BarrelDir, Vector End)
{
	local Vector X, Y, Z;
	local Vector FireDir;
	local Vector TraceStart, TraceEnd;
	local Actor HitActor;
	local Pawn HitPawn;
	local Vector HitLocation, HitNormal;

	GetAxes(BarrelDir, X, Y, Z);

	FireDir = Normal(End - Start) * Precision;
	FireDir += Vector(BarrelDir) * (1.0 - Precision);
	FireDir = Normal(FireDir);

	TraceStart = Start;
	TraceStart += X * FireInfo.FireOffset.X;
	TraceStart += Y * FireInfo.FireOffset.Y;
	TraceStart += Z * FireInfo.FireOffset.Z;

	TraceEnd = FireDir * FireInfo.MaxRange;
	TraceEnd += TraceStart;

	//Trace until something solid is hit.
	foreach TraceActors(class'Actor', HitActor, HitLocation, HitNormal, TraceEnd, TraceStart, vect(0.0, 0.0, 0.0),, TRACEFLAG_Bullet)
	{
		if(HitActor.bWorldGeometry)
		{
			break;
		}

		HitPawn = Pawn(HitActor);

		if(HitPawn == none || !Cannon.IsSameTeam(HitPawn))
		{
			HitActor.TakeDamage(FireInfo.Damage, Cannon.InstigatorController, HitLocation, FireInfo.Momentum * Normal(TraceEnd - TraceStart), FireInfo.DamageType,, Cannon);
		}
	}

	if(HitActor == none)
	{
		HitLocation	= TraceEnd;
	}

	super.FireAt(Start, BarrelDir, End);

	return true;
}

simulated function FlashMuzzleFlash()
{
	local Vector X, Y, Z;
	local Rotator BarrelDir;
	local Vector TraceStart, TraceEnd;
	local Vector HitLocation, HitNormal;

	Cannon.WeaponComponent.GetSocketWorldLocationAndRotation(MuzzleSocketName, TraceStart, BarrelDir);
	GetAxes(BarrelDir, X, Y, Z);

	TraceStart = Cannon.GetPawnViewLocation();
	TraceStart += X * FireInfo.FireOffset.X;
	TraceStart += Y * FireInfo.FireOffset.Y;
	TraceStart += Z * FireInfo.FireOffset.Z;

	TraceEnd = X * FireInfo.MaxRange;
	TraceEnd += TraceStart;

	if(Cannon.Trace(HitLocation, HitNormal, TraceEnd, TraceStart, false, vect(0,0,0),, TRACEFLAG_Bullet) == none)
	{
		HitLocation = TraceEnd;
	}

	/**
	 * The beam is a bit complicated because it needs a number of particles spawned evenly along an arbitrarily long line.
	 * The beam emitter has absolute start and end locations.
	 * The ring particles are emitted every x units traveled, by the SpawnPerUnit emitter module.
	 * In order for the emitter to consider itself to have moved, it has to take some time to reach its destination.
	 * To accomplish that, the movement of the beam is delayed with a timer.
	 */
	BeamEmitter = Spawn(class'EmitterSpawnable', Cannon,, TraceStart, BarrelDir);
	BeamEmitter.SetTemplate(BeamTemplate);
	BeamEmitter.SetVectorParameter('BeamStart', TraceStart);
	BeamEmitter.SetVectorParameter('BeamEnd', HitLocation);
	BeamEndLocation = HitLocation;
	SetTimer(BeamMoveDelay, false, 'MoveBeam');

	FireInfo.CheckBulletWhip(TraceStart, HitLocation);
	HitEffects.PlayImpactEffects(HitLocation - Normal(HitLocation - TraceStart), HitLocation, self);

	if(!WorldInfo.bDropDetail)
	{
		UTEmitterPool(WorldInfo.MyEmitterPool).SpawnExplosionLight(ImpactLightClass, HitLocation);
	}

	super.FlashMuzzleFlash();
}

simulated function MoveBeam()
{
	if(BeamEmitter != none)
	{
		BeamEmitter.SetLocation(BeamEndLocation);
	}
}

defaultproperties
{
	FriendlyName="MicroShock"
	Description="The result of recent advances in miniaturization technology applied to existing Enhanced Shock Rifle designs, the MicroShock is capable of unprecedented damage output for its size. Once the targeting issues have been overcome, the MicroShock is expected to revolutionize the automated defence market."

	Precision=0.5
	BeamMoveDelay=0.05

	WeaponMesh=SkeletalMesh'SentinelGear_Resources.Meshes.Weapons.SK_MicroShock'
	PhysicsAsset=PhysicsAsset'SentinelGear_Resources.Physics.SK_MicroShock_Physics'
	WeaponMaterial=MaterialInstance'SentinelGear_Resources.Materials.Weapons.MI_MicroShock'
	WeaponSpawnMaterial=MaterialInstance'SentinelGear_Resources.Materials.Weapons.MI_MicroShock_Spawn'
	WeaponDeadMaterial=MaterialInstance'SentinelGear_Resources.Materials.Weapons.MI_MicroShock_Dead'
	WeaponScale=1.6

	ActivateSound=SoundCue'SentinelGear_Resources.Sounds.Weapons.ShockRiflePickupCue'

	BeamTemplate=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_MicroShock_Beam'
	ImpactLightClass=class'UTExplosionLight_MicroShock'

	ExtraStrength=0.8
	AmmoCost=200
	CompatibleSentinels.Remove(class'Sentinel_Floor')

	Begin Object Class=SentinelWeaponComponent_FireInfo Name=FireInfo0
		FireInterval=2.0
		FireOffset=(X=25.0,Y=0.0,Z=0.0)
		Damage=300.0
		Momentum=50000.0
		DamageType=class'UTDmgType_MicroShock'
		MaxRange=20000.0
		Spread=0.0
		MaxAimError=1.0
		FireSound=SoundCue'SentinelGear_Resources.Sounds.Weapons.BShockRifleFireCue'
		BulletWhip=SoundCue'A_Weapon_ShockRifle.Cue.A_Weapon_SR_WhipCue'
		RotationStopTime=0.5
	End Object
	FireInfo=FireInfo0

	Begin Object Class=SentinelWeaponComponent_MuzzleFlash Name=MuzzleFlash0
		Template=ParticleSystem'WP_ShockRifle.Particles.P_Shockrifle_Instagib_3P_MF_Red'
		Scale=0.8
		MuzzleFlashDuration=0.2
		MuzzleFlashOffset=(X=0.0,Y=0.0,Z=0.0)
		MuzzleFlashColourName=Colour
	End Object
	MuzzleFlash=MuzzleFlash0

	Begin Object Class=SentinelWeaponComponent_MuzzleFlashLight Name=MuzzleFlashLightComponent0
		LightOffset=(X=40.0,Y=0.0,Z=0.0)
		TimeShift=((StartTime=0.0,Radius=400,Brightness=10,LightColor=(R=255,G=150,B=0,A=255)),(StartTime=0.6,Radius=100,Brightness=0,LightColor=(R=255,G=64,B=0,A=255)))
	End Object
	MuzzleFlashLight=MuzzleFlashLightComponent0

	Begin Object Class=SentinelWeaponComponent_HitEffects Name=HitEffectsComp
		ImpactEffects.Empty()
		DefaultImpactEffect=(Sound=SoundCue'A_Weapon_ShockRifle.Cue.A_Weapon_SR_AltFireImpactCue',ParticleTemplate=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_MicroShock_Impact',DecalMaterials=(none,none,none))
	End Object
	HitEffects=HitEffectsComp
}
